@@ -65,7 +65,7 @@ void RpczService::enable(::google::protobuf::RpcController* cntl_base,
     const bool use_html = UseHTML(cntl->http_request());
     cntl->http_response().set_content_type(
         use_html ? "text/html" : "text/plain");
-    if (!GFLAGS_NS::SetCommandLineOption("enable_rpcz", "true").empty()) {
+    if (!GFLAGS_NAMESPACE::SetCommandLineOption("enable_rpcz", "true").empty()) {
         if (use_html) {
             // Redirect to /rpcz
             cntl->response_attachment().append(
@@ -94,7 +94,7 @@ void RpczService::disable(::google::protobuf::RpcController* cntl_base,
     const bool use_html = UseHTML(cntl->http_request());
     cntl->http_response().set_content_type(
         use_html ? "text/html" : "text/plain");
-    if (!GFLAGS_NS::SetCommandLineOption("enable_rpcz", "false").empty()) {
+    if (!GFLAGS_NAMESPACE::SetCommandLineOption("enable_rpcz", "false").empty()) {
         if (use_html) {
             // Redirect to /rpcz
             cntl->response_attachment().append(
