@@ -69,11 +69,15 @@ int EventDispatcher::Start(const bthread_attr_t* thread_attr) {
 
     // Set _thread_attr before creating kqueue thread to make sure
     // everyting seems sane to the thread.
-    _thread_attr = (thread_attr ? *thread_attr : BTHREAD_ATTR_NORMAL);
+    if (thread_attr) {
+        _thread_attr = *thread_attr;
+    }
 
     //_thread_attr is used in StartInputEvent(), assign flag NEVER_QUIT to it will cause new bthread
     // that created by kevent() never to quit.
-    bthread_attr_t kqueue_thread_attr = _thread_attr | BTHREAD_NEVER_QUIT;
+    // Only event dispatcher thread has flag BTHREAD_GLOBAL_PRIORITY.
+    bthread_attr_t kqueue_thread_attr =
+        _thread_attr | BTHREAD_NEVER_QUIT | BTHREAD_GLOBAL_PRIORITY;
 
     // Polling thread uses the same attr for consumer threads (NORMAL right
     // now). Previously, we used small stack (32KB) which may be overflowed
