@@ -24,10 +24,26 @@
 #ifndef BUTIL_IOBUF_INL_H
 #define BUTIL_IOBUF_INL_H
 
+#include "butil/atomicops.h"                // butil::atomic
+#include "butil/thread_local.h"             // thread_atexit
+#include "butil/logging.h"                  // CHECK, LOG
+
 void* fast_memcpy(void *__restrict dest, const void *__restrict src, size_t n);
 
 namespace butil {
 
+using UserDataDeleter = std::function<void(void*)>;
+
+struct UserDataExtension {
+    UserDataDeleter deleter;
+};
+
+bool IsIOBufProfilerSamplable();
+void SubmitIOBufSample(IOBuf::Block* block, int64_t ref);
+
+const uint16_t IOBUF_BLOCK_FLAGS_USER_DATA = 1 << 0;
+const uint16_t IOBUF_BLOCK_FLAGS_SAMPLED = 1 << 1;
+
 inline ssize_t IOBuf::cut_into_file_descriptor(int fd, size_t size_hint) {
     return pcut_into_file_descriptor(fd, -1, size_hint);
 }
@@ -425,6 +441,212 @@ inline size_t IOBufBytesIterator::forward(size_t n) {
     return nc;
 }
 
+// Used by max_blocks_per_thread()
+bool IsIOBufProfilerEnabled();
+
+namespace iobuf {
+void inc_g_nblock();
+void dec_g_nblock();
+
+void inc_g_blockmem();
+void dec_g_blockmem();
+
+void inc_g_num_hit_tls_threshold();
+void dec_g_num_hit_tls_threshold();
+
+// Function pointers to allocate or deallocate memory for a IOBuf::Block
+extern void* (*blockmem_allocate)(size_t);
+extern void  (*blockmem_deallocate)(void*);
+
+} // namespace iobuf
+
+struct IOBuf::Block {
+    butil::atomic<int> nshared;
+    uint16_t flags;
+    uint16_t abi_check;  // original cap, never be zero.
+    uint32_t size;
+    uint32_t cap;
+    // When flag is 0, portal_next is valid.
+    // When flag & IOBUF_BLOCK_FLAGS_USER_DATA is non-0, data_meta is valid.
+    union {
+        Block* portal_next;
+        uint64_t data_meta;
+    } u;
+    // When flag is 0, data points to `size` bytes starting at `(char*)this+sizeof(Block)'
+    // When flag & IOBUF_BLOCK_FLAGS_USER_DATA is non-0, data points to the user data and
+    // the deleter is put in UserDataExtension at `(char*)this+sizeof(Block)'
+    char* data;
+        
+    Block(char* data_in, uint32_t data_size)
+        : nshared(1)
+        , flags(0)
+        , abi_check(0)
+        , size(0)
+        , cap(data_size)
+        , u({NULL})
+        , data(data_in) {
+        iobuf::inc_g_nblock();
+        iobuf::inc_g_blockmem();
+        if (is_samplable()) {
+            SubmitIOBufSample(this, 1);
+        }
+    }
+
+    Block(char* data_in, uint32_t data_size, UserDataDeleter deleter)
+        : nshared(1)
+        , flags(IOBUF_BLOCK_FLAGS_USER_DATA)
+        , abi_check(0)
+        , size(data_size)
+        , cap(data_size)
+        , u({0})
+        , data(data_in) {
+        auto ext = new (get_user_data_extension()) UserDataExtension();
+        ext->deleter = std::move(deleter);
+        if (is_samplable()) {
+            SubmitIOBufSample(this, 1);
+        }
+    }
+
+    // Undefined behavior when (flags & IOBUF_BLOCK_FLAGS_USER_DATA) is 0.
+    UserDataExtension* get_user_data_extension() {
+        char* p = (char*)this;
+        return (UserDataExtension*)(p + sizeof(Block));
+    }
+
+    inline void check_abi() {
+#ifndef NDEBUG
+    if (abi_check != 0) {
+        LOG(FATAL) << "Your program seems to wrongly contain two "
+            "ABI-incompatible implementations of IOBuf";
+    }
+#endif
+}
+
+    void inc_ref() {
+        check_abi();
+        nshared.fetch_add(1, butil::memory_order_relaxed);
+        if (sampled()) {
+            SubmitIOBufSample(this, 1);
+        }
+    }
+        
+    void dec_ref() {
+        check_abi();
+        if (sampled()) {
+            SubmitIOBufSample(this, -1);
+        }
+        if (nshared.fetch_sub(1, butil::memory_order_release) == 1) {
+            butil::atomic_thread_fence(butil::memory_order_acquire);
+            if (!is_user_data()) {
+                iobuf::dec_g_nblock();
+                iobuf::dec_g_blockmem();
+                this->~Block();
+                iobuf::blockmem_deallocate(this);
+            } else if (flags & IOBUF_BLOCK_FLAGS_USER_DATA) {
+                auto ext = get_user_data_extension();
+                ext->deleter(data);
+                ext->~UserDataExtension();
+                this->~Block();
+                free(this);
+            }
+        }
+    }
+
+    int ref_count() const {
+        return nshared.load(butil::memory_order_relaxed);
+    }
+
+    bool full() const { return size >= cap; }
+    size_t left_space() const { return cap - size; }
+
+private:
+    bool is_samplable() {
+        if (IsIOBufProfilerSamplable()) {
+            flags |= IOBUF_BLOCK_FLAGS_SAMPLED;
+            return true;
+        }
+        return false;
+    }
+
+    bool sampled() const {
+        return flags & IOBUF_BLOCK_FLAGS_SAMPLED;
+    }
+
+    bool is_user_data() const {
+        return flags & IOBUF_BLOCK_FLAGS_USER_DATA;
+    }
+};
+
+namespace iobuf {
+struct TLSData {
+    // Head of the TLS block chain.
+    IOBuf::Block* block_head;
+    
+    // Number of TLS blocks
+    int num_blocks;
+    
+    // True if the remote_tls_block_chain is registered to the thread.
+    bool registered;
+};
+
+// Max number of blocks in each TLS. This is a soft limit namely
+// release_tls_block_chain() may exceed this limit sometimes.
+const int MAX_BLOCKS_PER_THREAD = 8;
+
+inline int max_blocks_per_thread() {
+    // If IOBufProfiler is enabled, do not cache blocks in TLS.
+    return IsIOBufProfilerEnabled() ? 0 : MAX_BLOCKS_PER_THREAD;
+}
+
+TLSData* get_g_tls_data();
+void remove_tls_block_chain();
+
+IOBuf::Block* acquire_tls_block();
+
+// Return one block to TLS.
+inline void release_tls_block(IOBuf::Block* b) {
+    if (!b) {
+        return;
+    }
+    TLSData *tls_data = get_g_tls_data();
+    if (b->full()) {
+        b->dec_ref();
+    } else if (tls_data->num_blocks >= max_blocks_per_thread()) {
+        b->dec_ref();
+        // g_num_hit_tls_threshold.fetch_add(1, butil::memory_order_relaxed);
+        inc_g_num_hit_tls_threshold();
+    } else {
+        b->u.portal_next = tls_data->block_head;
+        tls_data->block_head = b;
+        ++tls_data->num_blocks;
+        if (!tls_data->registered) {
+            tls_data->registered = true;
+            butil::thread_atexit(remove_tls_block_chain);
+        }
+    }
+}
+
+inline IOBuf::Block* create_block(const size_t block_size) {
+    if (block_size > 0xFFFFFFFFULL) {
+        LOG(FATAL) << "block_size=" << block_size << " is too large";
+        return NULL;
+    }
+    char* mem = (char*)iobuf::blockmem_allocate(block_size);
+    if (mem == NULL) {
+        return NULL;
+    }
+    return new (mem) IOBuf::Block(mem + sizeof(IOBuf::Block),
+                                  block_size - sizeof(IOBuf::Block));
+}
+
+inline IOBuf::Block* create_block() {
+    return create_block(IOBuf::DEFAULT_BLOCK_SIZE);
+}
+
+void* cp(void *__restrict dest, const void *__restrict src, size_t n);
+
+};  // namespace iobuf;
+
 }  // namespace butil
 
-#endif  // BUTIL_IOBUF_INL_H
+#endif  // BUTIL_IOBUF_INL_H
\ No newline at end of file
