@@ -123,6 +123,10 @@ public:
 
     // Put name of all stats label into `names'
     void list_stats(std::vector<key_type>* names);
+
+    void set_max_stats_count(size_t max_stats_count) {
+        _max_stats_count = std::max(max_stats_count, max_stats_count);
+    }
     
 #ifdef UNIT_TEST
     // Get real bvar pointer object 
@@ -186,10 +190,6 @@ private:
     
     // Remove all stats so those not count and dump
     void delete_stats();
-
-    void set_max_stats_count(size_t max_stats_count) {
-        _max_stats_count = max_stats_count;
-    }
     
     static size_t init_flatmap(MetricMap& bg);
     
