@@ -34,51 +34,43 @@ static const std::string ALLOW_UNUSED METRIC_TYPE_SUMMARY = "summary";
 static const std::string ALLOW_UNUSED METRIC_TYPE_HISTOGRAM = "histogram";
 static const std::string ALLOW_UNUSED METRIC_TYPE_GAUGE = "gauge";
 
-template <typename T>
-inline
-MultiDimension<T>::MultiDimension(const key_type& labels)
+template <typename T, typename KeyType>
+MultiDimension<T, KeyType>::MultiDimension(const key_type& labels)
     : Base(labels)
-    , _max_stats_count(FLAGS_max_multi_dimension_stats_count)
-{
+    , _max_stats_count(FLAGS_max_multi_dimension_stats_count) {
     _metric_map.Modify(init_flatmap);
 }
 
-template <typename T>
-inline
-MultiDimension<T>::MultiDimension(const butil::StringPiece& name,
-                                  const key_type& labels)
-    : MultiDimension(labels)
-{
+template <typename T, typename KeyType>
+MultiDimension<T, KeyType>::MultiDimension(const butil::StringPiece& name,
+                                           const key_type& labels)
+    : MultiDimension(labels) {
     this->expose(name);
 }
 
-template <typename T>
-inline
-MultiDimension<T>::MultiDimension(const butil::StringPiece& prefix,
-                                  const butil::StringPiece& name,
-                                  const key_type& labels)
-    : MultiDimension(labels)
-{
+template <typename T, typename KeyType>
+MultiDimension<T, KeyType>::MultiDimension(const butil::StringPiece& prefix,
+                                           const butil::StringPiece& name,
+                                           const key_type& labels)
+    : MultiDimension(labels) {
     this->expose_as(prefix, name);
 }
 
-template <typename T>
-MultiDimension<T>::~MultiDimension() {
-    hide();
+template <typename T, typename KeyType>
+MultiDimension<T, KeyType>::~MultiDimension() {
+    this->hide();
     delete_stats();
 }
 
-template <typename T>
-inline
-size_t MultiDimension<T>::init_flatmap(MetricMap& bg) {
+template <typename T, typename KeyType>
+size_t MultiDimension<T, KeyType>::init_flatmap(MetricMap& bg) {
     // size = 1 << 13
     CHECK_EQ(0, bg.init(8192, 80));
     return (size_t)1;
 }
 
-template <typename T>
-inline
-size_t MultiDimension<T>::count_stats() {
+template <typename T, typename KeyType>
+size_t MultiDimension<T, KeyType>::count_stats() {
     MetricMapScopedPtr metric_map_ptr;
     if (_metric_map.Read(&metric_map_ptr) != 0) {
         LOG(ERROR) << "Fail to read dbd";
@@ -87,21 +79,17 @@ size_t MultiDimension<T>::count_stats() {
     return metric_map_ptr->size();
 }
 
-template <typename T>
-inline
-void MultiDimension<T>::delete_stats(const key_type& labels_value) {
+template <typename T, typename KeyType>
+template <typename K>
+void MultiDimension<T, KeyType>::delete_stats(const K& labels_value) {
     if (is_valid_lables_value(labels_value)) {
-        // Because there are two copies(foreground and background) in DBD, we need to use an empty tmp_metric,
-        // get the deleted value of second copy into tmp_metric, which can prevent the bvar object from being deleted twice.
+        // Because there are two copies(foreground and background) in DBD,
+        // we need to use an empty tmp_metric, get the deleted value of
+        // second copy into tmp_metric, which can prevent the bvar object
+        // from being deleted twice.
         op_value_type tmp_metric = NULL;
         auto erase_fn = [&labels_value, &tmp_metric](MetricMap& bg) {
-            auto it = bg.seek(labels_value);
-            if (it != NULL) {
-                tmp_metric = *it;
-                bg.erase(labels_value);
-                return 1;
-            }
-            return 0;
+            return bg.erase(labels_value, &tmp_metric);
         };
         _metric_map.Modify(erase_fn);
         if (tmp_metric) {
@@ -110,9 +98,8 @@ void MultiDimension<T>::delete_stats(const key_type& labels_value) {
     }
 }
 
-template <typename T>
-inline
-void MultiDimension<T>::delete_stats() {
+template <typename T, typename KeyType>
+void MultiDimension<T, KeyType>::delete_stats() {
     // Because there are two copies(foreground and background) in DBD, we need to use an empty tmp_map,
     // swap two copies with empty, and get the value of second copy into tmp_map,
     // then traversal tmp_map and delete bvar object,
@@ -133,9 +120,8 @@ void MultiDimension<T>::delete_stats() {
     }
 }
 
-template <typename T>
-inline
-void MultiDimension<T>::list_stats(std::vector<key_type>* names) {
+template <typename T, typename KeyType>
+void MultiDimension<T, KeyType>::list_stats(std::vector<key_type>* names) {
     if (names == NULL) {
         return;
     }
@@ -151,9 +137,9 @@ void MultiDimension<T>::list_stats(std::vector<key_type>* names) {
     }
 }
 
-template <typename T>
-inline
-T* MultiDimension<T>::get_stats_impl(const key_type& labels_value) {
+template <typename T, typename KeyType>
+template <typename K>
+T* MultiDimension<T, KeyType>::get_stats_impl(const K& labels_value) {
     if (!is_valid_lables_value(labels_value)) {
         return nullptr;
     }
@@ -170,28 +156,30 @@ T* MultiDimension<T>::get_stats_impl(const key_type& labels_value) {
     return (*it);
 }
 
-template <typename T>
-inline
-T* MultiDimension<T>::get_stats_impl(const key_type& labels_value, STATS_OP stats_op, bool* do_write) {
+template <typename T, typename KeyType>
+template <typename K>
+T* MultiDimension<T, KeyType>::get_stats_impl(
+    const K& labels_value, STATS_OP stats_op, bool* do_write) {
     if (!is_valid_lables_value(labels_value)) {
         return nullptr;
     }
     {
         MetricMapScopedPtr metric_map_ptr;
-        if (_metric_map.Read(&metric_map_ptr) != 0) {
+        if (0 != _metric_map.Read(&metric_map_ptr)) {
             LOG(ERROR) << "Fail to read dbd";
             return nullptr;
         }
 
         auto it = metric_map_ptr->seek(labels_value);
-        if (it != NULL) {
+        if (NULL != it) {
             return (*it);
         } else if (READ_ONLY == stats_op) {
             return nullptr;
         }
 
         if (metric_map_ptr->size() > _max_stats_count) {
-            LOG(ERROR) << "Too many stats seen, overflow detected, max stats count=" << _max_stats_count;
+            LOG(ERROR) << "Too many stats seen, overflow detected, max stats count="
+                       << _max_stats_count;
             return nullptr;
         }
     }
@@ -209,37 +197,35 @@ T* MultiDimension<T>::get_stats_impl(const key_type& labels_value, STATS_OP stat
         if (do_write) {
             *do_write = true;
         }
-        if (NULL != cache_metric) {
-            bg.insert(labels_value, cache_metric);
-        } else {
-            T* add_metric = new T();
-            bg.insert(labels_value, add_metric);
-            cache_metric = add_metric;
+
+        if (NULL == cache_metric) {
+            cache_metric = new T();
         }
+        insert_metrics_map(bg, labels_value, cache_metric);
         return 1;
     };
     _metric_map.Modify(insert_fn);
     return cache_metric;
 }
 
-template <typename T>
-inline
-void MultiDimension<T>::clear_stats() {
+template <typename T, typename KeyType>
+void MultiDimension<T, KeyType>::clear_stats() {
     delete_stats();
 }
 
-template <typename T>
-inline
-bool MultiDimension<T>::has_stats(const key_type& labels_value) {
+template <typename T, typename KeyType>
+template <typename K>
+bool MultiDimension<T, KeyType>::has_stats(const K& labels_value) {
     return get_stats_impl(labels_value) != nullptr;
 }
 
-template <typename T>
-inline
-size_t MultiDimension<T>::dump(Dumper* dumper, const DumpOptions* options) {
+template <typename T, typename KeyType>
+template <typename U>
+typename std::enable_if<!butil::is_same<LatencyRecorder, U>::value, size_t>::type
+MultiDimension<T, KeyType>::dump_impl(Dumper* dumper, const DumpOptions* options) {
     std::vector<key_type> label_names;
     list_stats(&label_names);
-    if (label_names.empty() || !dumper->dump_comment(name(), METRIC_TYPE_GAUGE)) {
+    if (label_names.empty() || !dumper->dump_comment(this->name(), METRIC_TYPE_GAUGE)) {
         return 0;
     }
     size_t n = 0;
@@ -260,9 +246,10 @@ size_t MultiDimension<T>::dump(Dumper* dumper, const DumpOptions* options) {
     return n;
 }
 
-template <>
-inline
-size_t MultiDimension<bvar::LatencyRecorder>::dump(Dumper* dumper, const DumpOptions*) {
+template <typename T, typename KeyType>
+template <typename U>
+typename std::enable_if<butil::is_same<LatencyRecorder, U>::value, size_t>::type
+MultiDimension<T, KeyType>::dump_impl(Dumper* dumper, const DumpOptions*) {
     std::vector<key_type> label_names;
     list_stats(&label_names);
     if (label_names.empty()) {
@@ -272,7 +259,7 @@ size_t MultiDimension<bvar::LatencyRecorder>::dump(Dumper* dumper, const DumpOpt
     // To meet prometheus specification, we must guarantee no second TYPE line for one metric name
 
     // latency comment
-    dumper->dump_comment(name() + "_latency", METRIC_TYPE_GAUGE);
+    dumper->dump_comment(this->name() + "_latency", METRIC_TYPE_GAUGE);
     for (auto &label_name : label_names) {
         bvar::LatencyRecorder* bvar = get_stats_impl(label_name);
         if (!bvar) {
@@ -310,7 +297,7 @@ size_t MultiDimension<bvar::LatencyRecorder>::dump(Dumper* dumper, const DumpOpt
     }
 
     // max_latency comment
-    dumper->dump_comment(name() + "_max_latency", METRIC_TYPE_GAUGE);
+    dumper->dump_comment(this->name() + "_max_latency", METRIC_TYPE_GAUGE);
     for (auto &label_name : label_names) {
         bvar::LatencyRecorder* bvar = get_stats_impl(label_name);
         if (!bvar) {
@@ -324,7 +311,7 @@ size_t MultiDimension<bvar::LatencyRecorder>::dump(Dumper* dumper, const DumpOpt
     }
 
     // qps comment
-    dumper->dump_comment(name() + "_qps", METRIC_TYPE_GAUGE);
+    dumper->dump_comment(this->name() + "_qps", METRIC_TYPE_GAUGE);
     for (auto &label_name : label_names) {
         bvar::LatencyRecorder* bvar = get_stats_impl(label_name);
         if (!bvar) {
@@ -338,7 +325,7 @@ size_t MultiDimension<bvar::LatencyRecorder>::dump(Dumper* dumper, const DumpOpt
     }
 
     // count comment
-    dumper->dump_comment(name() + "_count", METRIC_TYPE_COUNTER);
+    dumper->dump_comment(this->name() + "_count", METRIC_TYPE_COUNTER);
     for (auto &label_name : label_names) {
         bvar::LatencyRecorder* bvar = get_stats_impl(label_name);
         if (!bvar) {
@@ -353,29 +340,24 @@ size_t MultiDimension<bvar::LatencyRecorder>::dump(Dumper* dumper, const DumpOpt
     return n;
 }
 
-template <typename T>
-inline
-void MultiDimension<T>::make_dump_key(std::ostream& os, 
-                                      const key_type& labels_value,
-                                      const std::string& suffix,
-                                      const int quantile) {
-    os << name();
+template <typename T, typename KeyType>
+void MultiDimension<T, KeyType>::make_dump_key(std::ostream& os, const key_type& labels_value,
+                                               const std::string& suffix, int quantile) {
+    os << this->name();
     if (!suffix.empty()) {
         os << suffix;
     }
     make_labels_kvpair_string(os, labels_value, quantile);
 }
 
-template <typename T>
-inline
-void MultiDimension<T>::make_labels_kvpair_string(std::ostream& os, 
-                                                  const key_type& labels_value, 
-                                                  const int quantile) {
+template <typename T, typename KeyType>
+void MultiDimension<T, KeyType>::make_labels_kvpair_string(
+    std::ostream& os, const key_type& labels_value, int quantile) {
     os << "{";
-    auto label_key = _labels.cbegin();
+    auto label_key = this->_labels.cbegin();
     auto label_value = labels_value.cbegin();
     char comma[2] = {'\0', '\0'};
-    for (; label_key != _labels.cend() && label_value != labels_value.cend();
+    for (; label_key != this->_labels.cend() && label_value != labels_value.cend();
         label_key++, label_value++) {
         os << comma << label_key->c_str() << "=\"" << label_value->c_str() << "\"";
         comma[0] = ',';
@@ -386,22 +368,22 @@ void MultiDimension<T>::make_labels_kvpair_string(std::ostream& os,
     os << "}";
 }
 
-template <typename T>
-inline
-bool MultiDimension<T>::is_valid_lables_value(const key_type& labels_value) const {
-    if (count_labels() != labels_value.size()) {
-        LOG(ERROR) << "Invalid labels count";
+template <typename T, typename KeyType>
+template <typename K>
+bool MultiDimension<T, KeyType>::is_valid_lables_value(const K& labels_value) const {
+    if (this->count_labels() != labels_value.size()) {
+        LOG(ERROR) << "Invalid labels count" << this->count_labels()
+                   << " != " << labels_value.size();
         return false;
     }
     return true;
 }
 
-template <typename T>
-inline
-void MultiDimension<T>::describe(std::ostream& os) {
-    os << "{\"name\" : \"" << _name << "\", \"labels\" : [";
+template <typename T, typename KeyType>
+void MultiDimension<T, KeyType>::describe(std::ostream& os) {
+    os << "{\"name\" : \"" << this->name() << "\", \"labels\" : [";
     char comma[3] = {'\0', ' ', '\0'};
-    for (auto &label : _labels) {
+    for (auto& label : this->_labels) {
         os << comma << "\"" << label << "\"";
         comma[0] = ',';
     }
